#!/bin/bash
# We're executing some admin task as root (modifing perms)
CLUSTER_HOME=/var/opt/jfrog/cluster
ART_ETC_FOLDER=$ARTIFACTORY_HOME/etc

function checkEnv {
    local envVarName=$1
    eval local envVarValue=\$$envVarName
    if [ -z "$envVarValue" ]; then
        echo "Env variable $envVarName is not set, cannot start primary instance."
        exit 1
    fi
}

function checkAllEnvs {
    checkEnv ART_LICENSES
    checkEnv ART_DATABASE_CONNECTION_STRING
    checkEnv ART_DATABASE_USER
    checkEnv ART_DATABASE_PASSWORD
}

function setLicense {
    echo -n "$ART_LICENSES" | cut -d, -f1 > /etc/opt/jfrog/artifactory/artifactory.lic
}

function checkFolderPermissions {
    DIR=$1

    STAT=( $(stat -Lc "%A %G %U" $1) )
    PERM=${STAT[0]}
    GROUP=${STAT[1]}
    USER=${STAT[2]}

    if [[ $USER != "artifactory" ]] || [[ $GROUP != "artifactory"  ]] ; then
            if [[ ! $PERM =~ d......rw.  ]] ; then
                    echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions a+wr"
                    chmod a+wr $1
            fi
    elif [[ $USER == "artifactory" ]] && [[ ! $PERM =~ drw....... ]] ; then
            echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions u+wr"
            chmod u+wr $1
    elif [[ $GROUP == "artifactory" ]] && [[ ! $PERM =~ d...rw.... ]] ; then
            echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions g+wr"
            chmod g+wr $1
    fi
}

function configureDatabase {
    # Copy the mysql template if the file doesn't exist
    cp -n $ARTIFACTORY_HOME/misc/db/mysql.properties $CLUSTER_HOME/ha-etc/storage.properties
    # Replace url= with provided connection string...
    # Espace sed special chars & \ , 
    # ... before giving it to sed... (yes I know :|)
    local artDatabaseConnectionEscaped=$(echo $ART_DATABASE_CONNECTION_STRING | sed 's,[&\\,],\\&,g')
    local artUserNameEscaped=$(echo $ART_DATABASE_USER | sed 's,[&\\,],\\&,g')
    local artPasswordEscaped=$(echo $ART_DATABASE_PASSWORD | sed 's,[&\\,],\\&,g')
    sed -i -r -e "s,^url=.*$,url=$artDatabaseConnectionEscaped,g" $CLUSTER_HOME/ha-etc/storage.properties
    sed -i -r -e "s,^username=.*$,username=$artUserNameEscaped,g" $CLUSTER_HOME/ha-etc/storage.properties
    sed -i -r -e "s,^password=.*$,password=$artPasswordEscaped,g" $CLUSTER_HOME/ha-etc/storage.properties
    chown artifactory: $CLUSTER_HOME/ha-etc/storage.properties
}

function setPorts {
    # Artifactory's membership port, the PORT0 et PORT1 are specific to MESOS
    : ${MEMBERSHIP_PORT:=10042}
    : ${INSTANCE_PORT:=8081}
    if [ ! -z "$PORT0" ]; then
        INSTANCE_PORT=$PORT0
    fi
    if [ ! -z "$PORT1" ]; then
        MEMBERSHIP_PORT=$PORT1
    fi

    # Change the port
    sed -i -e 's,Connector port="\(.*\)",Connector port="'"$INSTANCE_PORT"'",g' /opt/jfrog/artifactory/tomcat/conf/server.xml
}

function setInstanceIp {
    # If no network is provided we take the first ip adress we found
    if [ -z "$ART_NETWORK" ]; then
        INSTANCE_IP=$(hostname -I|awk '{print $1}')
    # else we try to get it from the network provided
    else
        INSTANCE_IP=$(ip route show to match $ART_NETWORK | grep -Eo '[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}' | tail -1)
        if [ -z "$INSTANCE_IP" ]; then
            echo "[ERROR] Couldn't find a source IP routing to $ART_NETWORK, exiting" >&2
            exit 1
        fi
    fi
}

function setHaNodeProperties {
    cat >/etc/opt/jfrog/artifactory/ha-node.properties <<EOF
node.id="art-primary"
cluster.home=$CLUSTER_HOME
primary=true
context.url=http://$INSTANCE_IP:$INSTANCE_PORT/artifactory
membership.port=$MEMBERSHIP_PORT
EOF
    chown artifactory: /etc/opt/jfrog/artifactory/ha-node.properties
}

function setClusterHome {
    # Checking permissions for mounted volumes
    local artifactoryFolders=$(mount | grep artifactory | awk '{print $3}')
    for folder in $artifactoryFolders ; do
        checkFolderPermissions $folder
    done
    checkFolderPermissions $CLUSTER_HOME

    # Create the cluster home content the first time 
    if [ ! -f $CLUSTER_HOME/ha-etc/cluster.properties ]; then
        su artifactory -c "mkdir -p $CLUSTER_HOME/{ha-etc/{UI,plugins},ha-data/{filestore,tmp},ha-backup}"
        su artifactory -c "echo security.token=$(uuidgen) >$CLUSTER_HOME/ha-etc/cluster.properties"
    fi

    # Copy the licensesBucket plugin and nginx template into the cluster home (always do it in case of an update of the plugin in the image)
    su artifactory -c "cp -f /var/opt/jfrog/artifactory/etc/plugins/licensesBucket.groovy $CLUSTER_HOME/ha-etc/plugins/licensesBucket.groovy"
    su artifactory -c "cp -f /var/opt/jfrog/artifactory/etc/nginx.ftl $CLUSTER_HOME/ha-etc/nginx.ftl"
}

function waitForMysql {
   # Extract mysql hostname and port from the jdbc url
    local mysqlHostAndPort=$(echo $ART_DATABASE_CONNECTION_STRING | sed -e 's,^jdbc:mysql:\/\/\(.*\)$,\1:,g' | cut -d/ -f1)
    while ! curl -s "$mysqlHostAndPort" > /dev/null; do echo 'waiting for mysql'; sleep 3; done 
}

function setInitialConfiguration {
    # Artifactory's external server name
    : ${ART_SERVER_NAME:=artifactory-cluster}
    # Artifactory's port method, default to PORTPERREPO (can be SUBDOMAIN)
    : ${ART_REVERSE_PROXY_METHOD:=portPerRepo}
    if [ ! -f $CLUSTER_HOME/ha-etc/artifactory.config.bootstrap.xml ]; then
        sed -i -e "s,\[SERVERNAME\],$ART_SERVER_NAME,g" /tmp/artifactory.config.xml
        sed -i -e "s,\[RPMETHOD\],$ART_REVERSE_PROXY_METHOD,g" /tmp/artifactory.config.xml
        sed -i -e "s,\[PORT\],$INSTANCE_PORT,g" /tmp/artifactory.config.xml
        mv /tmp/artifactory.config.xml $CLUSTER_HOME/ha-etc/artifactory.config.import.xml
    fi
}

function restoreEtcFilesIfEmpty {
    #If etc folder doesn't exists OR exists but default file is missing, copies the default content from $ARTIFACTORY_HOME/defaults
    if [[ ! -d $ART_ETC_FOLDER ]] || [[ ! -e $ART_ETC_FOLDER/default ]] ; then
        echo "[WARN] $ART_ETC_FOLDER is either empty or missing default files, creating default files"
        cp -rpn $ARTIFACTORY_HOME/defaults/etc/* $ART_ETC_FOLDER/
    fi
}

function setDefaultJvmParams {
    local defaultFile=$ART_ETC_FOLDER/default
    # Change distrib type
    sed -i s/-Dartdist=rpm/-Dartdist=docker/g $defaultFile
    # In case of marathon, we can set the default Xmx with $MARATHON_APP_RESOURCE_MEM - 250M
    # We let this 250M to allow the non Heap Memory. If this wild guess is not enough, the user
    # can override all the jvm params with $JVM_OPTS
    if [ ! -z "$MARATHON_APP_RESOURCE_MEM" ]; then
        local maxMemory=$(expr $(echo "$MARATHON_APP_RESOURCE_MEM" | cut -d'.' -f 1) - 256)
        local minMemory=$(expr "$maxMemory" / 2)
        echo "[INFO] According to Marathon memory constraint of: $MARATHON_APP_RESOURCE_MEM, will set Xmx to ${maxMemory}m and Xms to ${minMemory}"

        local xmx="-Xms${minMemory}m -Xmx${maxMemory}m"
        local comment="# Adding Xmx"
        # Deleting previous set parameter
        sed -i -e "/^$comment/ { N; d; }" $defaultFile
        # Adding new parameter, don't change the form of this
        printf ''"$comment"' based on Marathon memory constraint \nexport JAVA_OPTIONS="$JAVA_OPTIONS '"$xmx"'"' >> $defaultFile
    fi
    if [ ! -z "$ART_RUNTIME_OPTS" ]; then
        echo "[INFO] Will add defaults JAVA_OPTIONS to $defaultFile with : $ART_RUNTIME_OPTS"
        printf '# Adding user provided options: \nexport JAVA_OPTIONS="$JAVA_OPTIONS '"$ART_RUNTIME_OPTS"'"' >> $defaultFile
    fi
}

checkAllEnvs
setLicense
setInstanceIp
setPorts
setClusterHome
setInitialConfiguration
configureDatabase
setHaNodeProperties
restoreEtcFilesIfEmpty
setDefaultJvmParams

# Wait and run as artifactory user
waitForMysql

exec su artifactory -m /opt/jfrog/artifactory/tomcat/bin/catalina.sh run
